# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Node Group API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class NodeGroup(object):
    """Identity Services Engine Node Group API (version: 3.1.1).

    Wraps the Identity Services Engine Node Group
    API and exposes the API as native Python
    methods that return native Python objects.



    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NodeGroup
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NodeGroup, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_node_groups(self,
                        headers=None,
                        **query_parameters):
        """This API retrieves the details of all the node groups in the
        cluster.  Each node group retrieved consists of name,
        description and MAR cache details like query-attempts,
        query-timeout, replication-attempts, replication-
        timeout. .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/deployment/node-group')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a0824f9a589c58cd8df522524cb550ad_v3_1_1', _api_response)

    def create_node_group(self,
                          description=None,
                          mar_cache=None,
                          name=None,
                          headers=None,
                          payload=None,
                          active_validation=True,
                          **query_parameters):
        """    This API creates a node group in the cluster.  A node group
        is a group of PSNs, where the PSNs maintain a heartbeat
        with each other. It is used primarily to terminate or
        transfer posture-pending sessions when a PSN in a local
        node group fails.  Node group members can communicate
        over TCP/7800.   The following parameters are used in
        the request body of the API:         PARAMETER
        DESCRIPTION   EXAMPLE           name * required   Name
        of the node group( valid-range:  1-100 characters)
        {"name": "site1"}       description   Description of the
        node group ( valid-range:  0-256 characters)   {"name":
        "site2", "description": "sample"}       query-attempts
        The number of times Cisco ISE attempts to perform the
        cache entry query. ( valid-range:  0 5,  default-value:
        1)   {"name": "site3","marCache": {"query-attempts": 1}}
        query-timeout   The time, in seconds, after which a
        cache entry query times out. ( valid-range:  1 10,
        default-value:  2) second(s)   {"name":
        "site4","marCache": {"query-timeout": 2}}
        replication-attempts   The number of times Cisco ISE
        attempts to perform MAR cache entry replication. (
        valid-range:  0 5,  default-value:  2)   {"name":
        "site5","marCache": {"replication-attempts": 2}}
        replication-timeout   The time, in seconds, after which
        the cache entry replication times out. ( valid-range:  1
        10,  default-value:  5) second(s)   {"name":
        "site6","marCache": {"replication-timeout": 5}}
        NOTE 1:  : Node group name and description cannot
        contain any of the following characters: ! % ^ : ; , . ~
        @ # & [ { ( | ) } ] ` > <  /   " + = ?  NOTE 2:  :
        Parameter marCache stands for Machine Access Restriction
        (MAR) cache that provides an additional means of
        controlling authorization for Active Directory-
        authentication users. We can enable the marCache for a
        nodegroup by providing key "marCache" in json request.
        Additionally we may also provide any combination of
        parameters query-attempts, query-timeout, replication-
        attempts, replication-timeout in marCache object. If no
        value is specified for a particular parameter its
        default value will be recorded.If no marCache object is
        given, marCache will be considered as disabled.    .

        Args:
            description(string): description, property of the
                request body.
            mar_cache(object): marCache, property of the request
                body.
            name(string): name, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'marCache':
                    mar_cache,
                'name':
                    name,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_d5efe180ef459b1a1d9f651e7c1eb92_v3_1_1')\
                .validate(_payload)

        e_url = ('/api/v1/deployment/node-group')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_d5efe180ef459b1a1d9f651e7c1eb92_v3_1_1', _api_response)

    def get_node_group(self,
                       node_group_name,
                       headers=None,
                       **query_parameters):
        """This API retrieves the details of a node group in the cluster
        using a node group name.

        Args:
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c9d1b776015057a59e659c8327ea91a1_v3_1_1', _api_response)

    def update_node_group(self,
                          node_group_name,
                          description=None,
                          mar_cache=None,
                          name=None,
                          headers=None,
                          payload=None,
                          active_validation=True,
                          **query_parameters):
        """    Purpose of this API is to update an existing node group.
        The following parameters are used in the request body of
        the API:         PARAMETER   DESCRIPTION   EXAMPLE
        name * required    Name of the node group existing in
        ISE( valid-range:  1-100 characters)   NOTE :  name of
        an existing node group cannot be modified. "name"
        parameter should be identical to the node group
        name(nodeGroupName) provided in the path     {"name":
        "site1"}       description    Description of the node
        group ( valid-range:  0-256 characters)     {"name":
        "site2", "description": "sample"}       query-attempts
        The number of times Cisco ISE attempts to perform the
        cache entry query. ( valid-range:  0 5,  default-value:
        1)   {"name": "site3","marCache": {"query-attempts": 1}}
        query-timeout   The time, in seconds, after which a
        cache entry query times out. ( valid-range:  1 10,
        default-value:  2) second(s)   {"name":
        "site4","marCache": {"query-timeout": 2}}
        replication-attempts   The number of times Cisco ISE
        attempts to perform MAR cache entry replication. (
        valid-range:  0 5,  default-value:  2)   {"name":
        "site5","marCache": {"replication-attempts": 2}}
        replication-timeout   The time, in seconds, after which
        the cache entry replication times out. ( valid-range:  1
        10,  default-value:  5) second(s)   {"name":
        "site6","marCache": {"replication-timeout": 5}}
        NOTE 1:   Node group name and description cannot contain
        any of the following characters: ! % ^ : ; , . ~ @ # & [
        { ( | ) } ] ` > <  /   " + = ?  NOTE 2:   Parameter
        marCache stands for Machine Access Restriction (MAR)
        cache that provides an additional means of controlling
        authorization for Active Directory-authentication users.
        We can enable the marCache for a nodegroup by providing
        key "marCache" in json request. Additionally we may also
        provide any combination of parameters query-attempts,
        query-timeout, replication-attempts, replication-timeout
        in marCache object. If no value is specified for a
        particular parameter its default value will be recorded.
        If no marCache object is given, marCache will be
        disabled.    .

        Args:
            description(string): description, property of the
                request body.
            mar_cache(object): marCache, property of the request
                body.
            name(string): name, property of the request body.
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'marCache':
                    mar_cache,
                'name':
                    name,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_fbd772420b8851349aa58fb4a9b006b8_v3_1_1')\
                .validate(_payload)

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_fbd772420b8851349aa58fb4a9b006b8_v3_1_1', _api_response)

    def delete_node_group(self,
                          force_delete,
                          node_group_name,
                          headers=None,
                          **query_parameters):
        """Delete an existing node group in the cluster. Deleting the node
        group does not delete the nodes, but failover is no
        longer carried out among the nodes.

        Args:
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            force_delete(bool): forceDelete query parameter. Force
                delete the group even if the node group
                contains one or more nodes.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(force_delete, bool,
                   may_be_none=False)
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
            'forceDelete':
                force_delete,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b2eacaba249e5762874a801f71631ae8_v3_1_1', _api_response)

    def add_node(self,
                 node_group_name,
                 hostname=None,
                 headers=None,
                 payload=None,
                 active_validation=True,
                 **query_parameters):
        """    This API adds a node to the node group in the cluster. When
        a node that belongs to a node group fails, another node
        in the same node group issues a Change of Authorization
        (CoA) for all the URL-redirected sessions on the failed
        node.   The following parameters are used in the request
        body of the API:         PARAMETER   DESCRIPTION
        EXAMPLE           hostname * required   Name of the host
        name    {"hostname": "isenode"}        .

        Args:
            hostname(string): hostname, property of the request
                body.
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'hostname':
                    hostname,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_fc44ec6afaf95ea9b51dd404abf46e4e_v3_1_1')\
                .validate(_payload)

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}/add-node')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_fc44ec6afaf95ea9b51dd404abf46e4e_v3_1_1', _api_response)

    def get_nodes(self,
                  node_group_name,
                  headers=None,
                  **query_parameters):
        """This API retrieves the list of nodes associated with a node
        group in the cluster with a given node group name.

        Args:
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}/node')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fe420544d425d3d873632dbb6c1b8c2_v3_1_1', _api_response)

    def remove_node(self,
                    node_group_name,
                    hostname=None,
                    headers=None,
                    payload=None,
                    active_validation=True,
                    **query_parameters):
        """    Purpose of this API is to remove a node from a node group in
        the cluster. Removing node from the node group does not
        delete the node, but failover is no longer carried out
        if the node is not part any node group.   The following
        parameters are used in the request body of the API:
        PARAMETER   DESCRIPTION   EXAMPLE           hostname *
        required   Name of the host name    {"hostname":
        "isenode"}        .

        Args:
            hostname(string): hostname, property of the request
                body.
            node_group_name(basestring): nodeGroupName path
                parameter. Name of the existing node
                group.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(node_group_name, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'nodeGroupName': node_group_name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'hostname':
                    hostname,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b1a343c45952a79d0bbfbadb02002b_v3_1_1')\
                .validate(_payload)

        e_url = ('/api/v1/deployment/node-group/{nodeGroupName}/remove-'
                 + 'node')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_b1a343c45952a79d0bbfbadb02002b_v3_1_1', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Network Access - Policy Set API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class NetworkAccessPolicySet(object):
    """Identity Services Engine Network Access - Policy Set API (version: 3.1.0).

    Wraps the Identity Services Engine Network Access - Policy Set
    API and exposes the API as native Python
    methods that return native Python objects.



    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NetworkAccessPolicySet
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NetworkAccessPolicySet, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_network_access_policy_sets(self,
                                       headers=None,
                                       **query_parameters):
        """Get all network access policy sets.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/policy/network-access/policy-set')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ed1ef503c091506aa8e446182e625365_v3_1_0', _api_response)

    def get_all(self,
                headers=None,
                **query_parameters):
        """Alias for `get_network_access_policy_sets <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.get_network_access_policy_sets>`_
        """
        return self.get_network_access_policy_sets(
            headers=headers,
            **query_parameters
        )

    def create_network_access_policy_set(self,
                                         condition=None,
                                         default=None,
                                         description=None,
                                         hit_counts=None,
                                         id=None,
                                         is_proxy=None,
                                         link=None,
                                         name=None,
                                         rank=None,
                                         service_name=None,
                                         state=None,
                                         headers=None,
                                         payload=None,
                                         active_validation=True,
                                         **query_parameters):
        """Network Access Create a new policy set:     Policy must include
        name , service identifier (either server sequence or
        allowed protocol) and a condition.     Condition has
        hierarchical structure which define a set of condition
        for which policy could be match.     Condition can be
        either reference to a stored Library condition, using
        model  ConditionReference , or, dynamically built
        conditions which are not stored in the conditions
        Library, using models  ConditionAttributes,
        ConditionAndBlock, ConditionOrBlock .    .

        Args:
            condition(object): condition, property of the request
                body.
            default(boolean): Flag which indicates if this policy
                set is the default one, property of the
                request body.
            description(string): The description for the policy set,
                property of the request body.
            hit_counts(integer): The amount of times the policy was
                matched, property of the request body.
            id(string): Identifier for the policy set, property of
                the request body.
            is_proxy(boolean): Flag which indicates if the policy
                set service is of type 'Proxy Sequence'
                or 'Allowed Protocols', property of the
                request body.
            link(object): link, property of the request body.
            name(string): Given name for the policy set, [Valid
                characters are alphanumerics,
                underscore, hyphen, space, period,
                parentheses], property of the request
                body.
            rank(integer): The rank(priority) in relation to other
                policy set. Lower rank is higher
                priority., property of the request body.
            service_name(string): Policy set service identifier
                Allowed Protocols,Server Sequence..,
                property of the request body.
            state(string): The state that the policy set is in. A
                disabled policy set cannot be matched.,
                property of the request body. Available
                values are 'disabled', 'enabled' and
                'monitor'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'condition':
                    condition,
                'default':
                    default,
                'description':
                    description,
                'hitCounts':
                    hit_counts,
                'id':
                    id,
                'isProxy':
                    is_proxy,
                'link':
                    link,
                'name':
                    name,
                'rank':
                    rank,
                'serviceName':
                    service_name,
                'state':
                    state,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_dfe1db8729d541fb3a17d31d47d1881_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/policy/network-access/policy-set')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_dfe1db8729d541fb3a17d31d47d1881_v3_1_0', _api_response)

    def create(self,
               condition=None,
               default=None,
               description=None,
               hit_counts=None,
               id=None,
               is_proxy=None,
               link=None,
               name=None,
               rank=None,
               service_name=None,
               state=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `create_network_access_policy_set <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.create_network_access_policy_set>`_
        """
        return self.create_network_access_policy_set(
            condition=condition,
            default=default,
            description=description,
            hit_counts=hit_counts,
            id=id,
            is_proxy=is_proxy,
            link=link,
            name=name,
            rank=rank,
            service_name=service_name,
            state=state,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def reset_hit_counts_network_access_policy_sets(self,
                                                    headers=None,
                                                    **query_parameters):
        """Network Access Reset HitCount for PolicySets.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/policy/network-access/policy-set/reset-hitcount')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fb6c1b3f335dbf8176a29e30eb6333_v3_1_0', _api_response)

    def reset_hit_counts(self,
                         headers=None,
                         **query_parameters):
        """Alias for `reset_hit_counts_network_access_policy_sets <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.reset_hit_counts_network_access_policy_sets>`_
        """
        return self.reset_hit_counts_network_access_policy_sets(
            headers=headers,
            **query_parameters
        )

    def get_network_access_policy_set_by_id(self,
                                            id,
                                            headers=None,
                                            **query_parameters):
        """Network Access Get policy set attributes.

        Args:
            id(basestring): id path parameter. Policy id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/api/v1/policy/network-access/policy-set/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d9f17adde53e2a08a650b9fe1714c_v3_1_0', _api_response)

    def get_by_id(self,
                  id,
                  headers=None,
                  **query_parameters):
        """Alias for `get_network_access_policy_set_by_id <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.get_network_access_policy_set_by_id>`_
        """
        return self.get_network_access_policy_set_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def update_network_access_policy_set_by_id(self,
                                               id,
                                               condition=None,
                                               default=None,
                                               description=None,
                                               hit_counts=None,
                                               is_proxy=None,
                                               link=None,
                                               name=None,
                                               rank=None,
                                               service_name=None,
                                               state=None,
                                               headers=None,
                                               payload=None,
                                               active_validation=True,
                                               **query_parameters):
        """Network Access Update a policy set.

        Args:
            condition(object): condition, property of the request
                body.
            default(boolean): Flag which indicates if this policy
                set is the default one, property of the
                request body.
            description(string): The description for the policy set,
                property of the request body.
            hit_counts(integer): The amount of times the policy was
                matched, property of the request body.
            id(string): Identifier for the policy set, property of
                the request body.
            is_proxy(boolean): Flag which indicates if the policy
                set service is of type 'Proxy Sequence'
                or 'Allowed Protocols', property of the
                request body.
            link(object): link, property of the request body.
            name(string): Given name for the policy set, [Valid
                characters are alphanumerics,
                underscore, hyphen, space, period,
                parentheses], property of the request
                body.
            rank(integer): The rank(priority) in relation to other
                policy set. Lower rank is higher
                priority., property of the request body.
            service_name(string): Policy set service identifier
                Allowed Protocols,Server Sequence..,
                property of the request body.
            state(string): The state that the policy set is in. A
                disabled policy set cannot be matched.,
                property of the request body. Available
                values are 'disabled', 'enabled' and
                'monitor'.
            id(basestring): id path parameter. Policy id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'condition':
                    condition,
                'default':
                    default,
                'description':
                    description,
                'hitCounts':
                    hit_counts,
                'id':
                    id,
                'isProxy':
                    is_proxy,
                'link':
                    link,
                'name':
                    name,
                'rank':
                    rank,
                'serviceName':
                    service_name,
                'state':
                    state,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_ff0055f9ef115a42bea6ffdd8e57d41b_v3_1_0')\
                .validate(_payload)

        e_url = ('/api/v1/policy/network-access/policy-set/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_ff0055f9ef115a42bea6ffdd8e57d41b_v3_1_0', _api_response)

    def update_by_id(self,
                     id,
                     condition=None,
                     default=None,
                     description=None,
                     hit_counts=None,
                     is_proxy=None,
                     link=None,
                     name=None,
                     rank=None,
                     service_name=None,
                     state=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Alias for `update_network_access_policy_set_by_id <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.update_network_access_policy_set_by_id>`_
        """
        return self.update_network_access_policy_set_by_id(
            id=id,
            condition=condition,
            default=default,
            description=description,
            hit_counts=hit_counts,
            is_proxy=is_proxy,
            link=link,
            name=name,
            rank=rank,
            service_name=service_name,
            state=state,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def delete_network_access_policy_set_by_id(self,
                                               id,
                                               headers=None,
                                               **query_parameters):
        """Network Access Delete a policy set.

        Args:
            id(basestring): id path parameter. Policy id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, basestring,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/api/v1/policy/network-access/policy-set/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fd4b5a56f8bd5f8f919e9fffc172e72f_v3_1_0', _api_response)

    def delete_by_id(self,
                     id,
                     headers=None,
                     **query_parameters):
        """Alias for `delete_network_access_policy_set_by_id <#ciscoisesdk.
        api.v3_1_0.network_access_policy_set.
        NetworkAccessPolicySet.delete_network_access_policy_set_by_id>`_
        """
        return self.delete_network_access_policy_set_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine SupportBundleDownload API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)
from ...pagination import get_next_page


class SupportBundleDownload(object):
    """Identity Services Engine SupportBundleDownload API (version: 3.1.1).

    Wraps the Identity Services Engine SupportBundleDownload
    API and exposes the API as native Python
    methods that return native Python objects.

    | Support Bundle Download API allows client to download support bundles provided those are generated and available in the default location.

    **Revision History**

    +----------------+----------------------+-----------------------+---------------------------+---+---+
    | **Revision #** | **Resource Version** | **Cisco ISE Version** | **Description**           |   |   |
    +----------------+----------------------+-----------------------+---------------------------+---+---+
    | 0              | 1.0                  | 2.7                   | Initial Cisco ISE Version |   |   |
    +----------------+----------------------+-----------------------+---------------------------+---+---+

    |

    **Resource Definition**

    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+
    | **Attribute** | **Type** | **Required** | **Description**                                                  | **Example Values** |
    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+
    | name          | String   | Yes          | Resource Name                                                    |                    |
    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+
    | id            | String   | No           | Resource UUID, mandatory for update                              |                    |
    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+
    | description   | String   | No           |                                                                  |                    |
    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+
    | fileName      | String   | Yes          | File name along with path from where file needs to be downloaded | filename           |
    +---------------+----------+--------------+------------------------------------------------------------------+--------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new SupportBundleDownload
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(SupportBundleDownload, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def download_support_bundle(self,
                                file_name=None,
                                dirpath=None,
                                save_file=None,
                                headers=None,
                                payload=None,
                                active_validation=True,
                                **query_parameters):
        """This API allows the client to upload a support bundle.

        Args:
            file_name(string): fileName, property of the request
                body.
            dirpath(basestring): Directory absolute path. Defaults to
                os.getcwd().
            save_file(bool): Enable or disable automatic file creation of
                raw response.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            urllib3.response.HTTPResponse: HTTP Response container. For more
            information check the `urlib3 documentation <https://urllib3.readthedocs.io/en/latest/reference/urllib3.response.html>`_

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
            DownloadFailure: If was not able to download the raw
            response to a file.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           basestring)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           basestring)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, basestring)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'fileName':
                    file_name,
            }
            _payload = {
                'ErsSupportBundleDownload': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_6d125b968b9d362a3458621d_v3_1_1')\
                .validate(_payload)

        e_url = ('/ers/config/supportbundledownload')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              stream=True, dirpath=dirpath, save_file=save_file,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              stream=True, dirpath=dirpath, save_file=save_file,
                                              **request_params)

        return self._object_factory('bpm_6d125b968b9d362a3458621d_v3_1_1', _api_response)

    def download(self,
                 file_name=None,
                 dirpath=None,
                 save_file=None,
                 headers=None,
                 payload=None,
                 active_validation=True,
                 **query_parameters):
        """Alias for `download_support_bundle <#ciscoisesdk.
        api.v3_1_1.support_bundle_download.
        SupportBundleDownload.download_support_bundle>`_
        """
        return self.download_support_bundle(
            file_name=file_name,
            dirpath=dirpath,
            save_file=save_file,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the support bundle download.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           basestring, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/supportbundledownload/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c57752629f546fb86e84c59285350f_v3_1_1', _api_response)
